{ open Parser
	open Buffer
	open Lexing
}

rule token = parse
  [' ' '\t' '\r'] { token lexbuf } (* Whitespace *)
| ['\n']   { new_line lexbuf ; token lexbuf }
| "[*"     { comment lexbuf }           (* Comments *)
| '"'	   { let buffer = Buffer.create 16 in
		STRING (stringr buffer lexbuf)} (* Quoted string *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| '['	   { LBLOCK }
| ']'	   { RBLOCK }
| ';'      { SEMI }
| ','	   { COMMA }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '%'	   { MOD }
| '='      { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| '!'	   { NEG }
| '^'	   { CONCAT }
| "->"	   { TRAN }
| "if"     { IF }
| "else if"{ ELSEIF }
| "else"   { ELSE }
| "repeat" { REPEAT }
| "var"	   { VAR }
| "true"|"false" as lxm { BOOLEAN(bool_of_string lxm) }
| "and"	   { AND }
| "or"	   { OR }
| "question" { QUE }
| "execute"  { EXC }
| "function" { FUNC }
| "return"   { RET }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*]" { token lexbuf }
| _    { comment lexbuf }

and stringr buffer = parse
  '"'       { Buffer.contents buffer }
| eof       { raise (Failure("end of file. end of string expected.")) }
| _ as char { Buffer.add_char buffer char; stringr buffer lexbuf }

